/*
	AC 3-Phase - An AC 3-Phase Calculator
	Copyright © 2007 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
	
	AC 3-Phase - version 1.0
	20 November, 2007
	Copyright © 2007 Harry Whitfield
	mailto:g6auc@arrl.net
*/

		include("menu.js");

		var minWidth = 180;
		var midWidth = 310;
		var maxWidth = 565;
		
		var curWidth  = minWidth;
		var userWidth = minWidth;

		var debug = false;
		
		var redButtonWithin = false;
		var amberButtonWithin = false;
		var greenButtonWithin = false;
				
		var hasFocus = false;
		
		var mode     = Number(preferences.modePref.value);
		var lineMode = Number(preferences.linePref.value);

		var paleGreen = "#CCFFCC";
		var white     = "#FFFFFF";
		
		var modeText = ['Voltage','Current','Power Factor','Power'];
		var lineText = ['Line Mode','Phase Mode'];
		var dockText = ['Line Mode','Phase Mode'];

		var modeValue = [14,13,11,7];
		var modeIndex = [-1,-1,-1,-1,-1,-1,-1,3,-1,-1,-1,2,-1,1,0];
		
		var multiplier;

		function parseNonNegFloat(s)
		{
			var v = parseFloat(s);
			if (isNaN(v)) { return v; }
			if (v >= 0)   { return v; }
			return NaN;
		}

		function parsePfFloat(s)
		{
			var v = parseFloat(s);
			if (isNaN(v)) { return v; }
			if ((v >= 0) && (v <= 1))    { return v; }
			return NaN;
		}
		
		function process()
		{
			var V, I, pf, P, result; 
			
			switch (mode)	// P=8, pf=4, C=2, V=1
			{
				case 7: V = parseNonNegFloat(voltage.data);
						I = parseNonNegFloat(current.data);
						pf= parsePfFloat(powerFactor.data);
						result = multiplier * V * I * pf; print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						power.data = result;
						break;
			
				case 11:V = parseNonNegFloat(voltage.data);
						I = parseNonNegFloat(current.data);
						P = parseNonNegFloat(power.data);
						result = P / (multiplier * V * I); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						if (result > 1.0) { result = 'Invalid Output '+ result; }
						powerFactor.data = result;
						break;
						
				case 13:V = parseNonNegFloat(voltage.data);
						pf= parsePfFloat(powerFactor.data);
						P = parseNonNegFloat(power.data);
						result = P / (multiplier * V * pf); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						current.data = result;
						break;

				case 14:I = parseNonNegFloat(current.data);
						pf= parsePfFloat(powerFactor.data);
						P = parseNonNegFloat(power.data);
						result = P / (multiplier * I * pf); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						voltage.data = result;
						break;

				default:beep();
						break;
			}
		}
		
		function setAmber(signed)
		{
			if (signed)
			{
				amberButton.src = "Resources/amberMinus.png";
			}
			else
			{
				amberButton.src = "Resources/amber.png";
			}
		}

		function normalButtons()
		{
			redButton.src   = "Resources/red.png";
			greenButton.src = "Resources/green.png";
			setAmber(false);
		}

		function activeButtons()
		{
			redButton.src   = "Resources/redCross.png";
			greenButton.src = "Resources/greenPlus.png";
			setAmber(true);
		}

		function focusButtons()
		{
			if (hasFocus)
			{
				redButton.src   = "Resources/red.png";
				greenButton.src = "Resources/green.png";
				setAmber(false);
			}
			else
			{
				redButton.src   = "Resources/grey.png";
				amberButton.src = "Resources/grey.png";
				greenButton.src = "Resources/grey.png";
			}
		}

		function toggleSize()
		{
			//print('main_window.width=' + main_window.width);
			
			if (main_window.width == userWidth)
			{
				setUp(midWidth);
			}
			else
			{
				setUp(userWidth);
			}
			if (network_window.visible)
			{
				network_window.hOffset = main_window.hOffset + main_window.width + 10;
				network_window.vOffset = main_window.vOffset;
			}
		}

		function setUp(width)
		{
			if (debug)
			{
				print('setUp():======== start ========:');
				print('width=' + width);
				print('minWidth=' + minWidth);
				print('maxWidth=' + maxWidth);
			}
			
			if (width < minWidth) { width = minWidth; }
			if (width > maxWidth) { width = maxWidth; }

			main_window.width = width;
			middle.width = width - 40;
			right.hOffset = width - 20;
			
			voltage.width = width - 30;
			current.width = width - 30;
			powerFactor.width = width - 30;
			power.width = width - 30;
						
			dragbox.hOffset = width - 15;
			
			if (network_window.visible)
			{
				network_window.hOffset = main_window.hOffset + main_window.width + 10;
				network_window.vOffset = main_window.vOffset;
			}

			updateNow();
			
			if (debug)
			{
				print('width=' + width);
		
				print('main_window.width=' + main_window.width);
				print('middle.width=' + middle.width);
				print('right.hOffset=' + right.hOffset);
			
				print('voltage.width=' + voltage.width);
				print('current.width=' + current.width);
				print('powerFactor.width=' + powerFactor.width);
				print('power.width=' + power.width);
				
				print('dragbox.hOffset=' + dragbox.hOffset);

				print('setUp():========  end  ========:');
			}

			return width;
		}
		
		function clearData()
		{
			voltage.data = "";
			current.data = "";
			powerFactor.data = "";
			power.data = "";
		}
		
		function showDockImage()
		{
			if (lineMode === 0)
			{
				buildVitality("Resources/star.png",  224, 200, dockText[lineMode]);
			}
			else
			{
				buildVitality("Resources/delta.png", 224, 200, dockText[lineMode]);
			}
		}

		function updatePrefs()
		{
			mode = Number(preferences.modePref.value);
			chooseMenu.setTick(modeText[modeIndex[mode]]);

			lineMode = Number(preferences.linePref.value);
			inputMenu.setTick(lineText[lineMode]);
			
			if (lineMode === 0)
			{
				multiplier = Math.sqrt(3.0);
				voltageHeading.data = "Line Voltage (V)";
				currentHeading.data = "Line Current (I)";
				buildVitality("Resources/star.png",  224, 200, dockText[lineMode]);
			}
			else
			{
				multiplier = 3.0;
				voltageHeading.data = "Phase Voltage (Vph)";
				currentHeading.data = "Phase Current (Iph)";
				buildVitality("Resources/delta.png", 224, 200, dockText[lineMode]);
			}

			userWidth = curWidth;
			
			voltage.font = current.font = powerFactor.font = power.font =  preferences.generalFontPref.value;
			paleGreen = preferences.bgColorPref.value;
			
			voltage.editable     = Boolean(mode & 1); if (mode & 1) { voltage.bgColor     = paleGreen; } else { voltage.bgColor     = white; }
			current.editable     = Boolean(mode & 2); if (mode & 2) { current.bgColor     = paleGreen; } else { current.bgColor     = white; }
			powerFactor.editable = Boolean(mode & 4); if (mode & 4) { powerFactor.bgColor = paleGreen; } else { powerFactor.bgColor = white; }
			power.editable       = Boolean(mode & 8); if (mode & 8) { power.bgColor       = paleGreen; } else { power.bgColor       = white; }
			
			clearData();

			if (mode & 1)
			{
				voltage.focus();
				voltage.select(0, -1);
			}
			else if (mode & 2)
			{
				current.focus();
				current.select(0, -1);
			}
			else if (mode & 4)
			{
				powerFactor.focus();
				powerFactor.select(0, -1);
			}
			else if (mode & 8)
			{
				power.focus();
				power.select(0, -1);
			}
			
			setAmber(false);
			setUp(curWidth);
		}
		
		function chooseMenuAction()								//	(i, data)	// define the action function for chooseMenu
		{
    		var i = this.i, data = this.data;
    		//print("Index:" + i + " Data:" + data);
    		preferences.modePref.value = String(modeValue[i]);
    		updatePrefs();
		}

		function inputMenuAction()								//	(i, data)	// define the action function for inputMenu
		{
    		var i = this.i, data = this.data;
    		//print("Index:" + i + " Data:" + data);
    		preferences.linePref.value = String(i);
    		updatePrefs();
		}

		var menuStringsArray = modeText; 						// define the array of menuStrings
		var selectedString   = modeText[modeIndex[mode]];  		// define the initially selected string, or set to ""
		var menuActionFunction   = chooseMenuAction;  			// define the name of the menuAction function - must be a function
		var menuImage = amberButton;

		var chooseMenu = new Menu(menuImage, menuStringsArray, selectedString, menuActionFunction); // call the menu constructor

		menuStringsArray = lineText; 					// define the array of menuStrings
		selectedString   = lineText[lineMode];  		// define the initially selected string, or set to ""
		menuActionFunction   = inputMenuAction;  		// define the name of the menuAction function - must be a function
		menuImage = greenButton;

		var inputMenu = new Menu(menuImage, menuStringsArray, selectedString, menuActionFunction); // call the menu constructor
		updatePrefs();
		setUp(curWidth);
		focusWidget();
